@@ -1,5 +1,11 @@
 from decimal import Decimal as MyDecimal, ROUND_HALF_EVEN
-import urlparse
+import six
+try:
+    from urlparse import urlparse, urlunparse
+except ImportError:
+    # python3
+    from urllib.parse import urlparse, urlunparse
+
 from flask_restful import types, marshal
 from flask import url_for
 
@@ -15,7 +21,7 @@ class MarshallingException(Exception):
     def __init__(self, underlying_exception):
         # just put the contextual representation of the error to hint on what
         # went wrong without exposing internals
-        super(MarshallingException, self).__init__(unicode(underlying_exception))
+        super(MarshallingException, self).__init__(six.text_type(underlying_exception))
 
 
 def is_indexable_but_not_string(obj):
@@ -24,6 +30,21 @@ def is_indexable_but_not_string(obj):
 
 def get_value(key, obj, default=None):
     """Helper for pulling a keyed value off various types of objects"""
+    if type(key) == int:
+        return _get_value_for_key(key, obj, default)
+    else:
+        return _get_value_for_keys(key.split('.'), obj, default)
+
+
+def _get_value_for_keys(keys, obj, default):
+    if len(keys) == 1:
+        return _get_value_for_key(keys[0], obj, default)
+    else:
+        return _get_value_for_keys(
+            keys[1:], _get_value_for_key(keys[0], obj, default), default)
+
+
+def _get_value_for_key(key, obj, default):
     if is_indexable_but_not_string(obj):
         try:
             return obj[key]
@@ -104,13 +125,11 @@ class Nested(Raw):
         super(Nested, self).__init__(**kwargs)
 
     def output(self, key, obj):
-        data = to_marshallable_type(obj)
-
-        attr = key if self.attribute is None else self.attribute
-        if self.allow_null and data.get(attr) is None:
+        value = get_value(key if self.attribute is None else self.attribute, obj)
+        if self.allow_null and value is None:
             return None
 
-        return marshal(data[attr], self.nested)
+        return marshal(value, self.nested)
 
 class List(Raw):
     def __init__(self, cls_or_instance):
@@ -136,13 +155,16 @@ class List(Raw):
             return [self.container.output(idx, value) for idx, val
                     in enumerate(value)]
 
+        if value is None:
+            return self.default
+
         return [marshal(value, self.container.nested)]
 
 
 class String(Raw):
     def format(self, value):
         try:
-            return unicode(value)
+            return six.text_type(value)
         except ValueError as ve:
             raise MarshallingException(ve)
 
@@ -168,7 +190,7 @@ class Boolean(Raw):
 class FormattedString(Raw):
     def __init__(self, src_str):
         super(FormattedString, self).__init__()
-        self.src_str = unicode(src_str)
+        self.src_str = six.text_type(src_str)
 
     def output(self, key, obj):
         try:
@@ -189,8 +211,8 @@ class Url(Raw):
     def output(self, key, obj):
         try:
             data = to_marshallable_type(obj)
-            o = urlparse.urlparse(url_for(self.endpoint, **data))
-            return urlparse.urlunparse(("", "", o.path, "", "", ""))
+            o = urlparse(url_for(self.endpoint, **data))
+            return urlunparse(("", "", o.path, "", "", ""))
         except TypeError as te:
             raise MarshallingException(te)
 
@@ -215,7 +237,7 @@ class Arbitrary(Raw):
     """
 
     def format(self, value):
-        return unicode(MyDecimal(value))
+        return six.text_type(MyDecimal(value))
 
 
 class DateTime(Raw):
@@ -238,6 +260,6 @@ class Fixed(Raw):
         dvalue = MyDecimal(value)
         if not dvalue.is_normal() and dvalue != ZERO:
             raise MarshallingException('Invalid Fixed precision number.')
-        return unicode(dvalue.quantize(self.precision, rounding=ROUND_HALF_EVEN))
+        return six.text_type(dvalue.quantize(self.precision, rounding=ROUND_HALF_EVEN))
 
 Price = Fixed
